unit Player;

interface

uses eXgine, SysUtils, Math, MyInterface, EngineUtils, Consts, SpriteEngine, OpenGL, Enemy, level;

const JumpWidth          = 128;
      SpeedJump          = 3;
      SpeedDownMax       = 4;
      PlayerW            = 64;
      PlayerH            = 64;
      PlayerFrameMax     = 8;
      PlayerHandFrameMax = 16;
      PlayerMaxReload    = 50;
      PlayerMaxHealth    = 3;
      PlayerSpeed        = 2;
      ScreenMoveLimit    = 200;
      ScoreActiveTime    = 100;
      EffectFrameSpeed   = 0.1;
      MaxTimeNoKill      = 80;
      SnowBallAngleSpeed = 2;
      SnowBallSpeed      = 4;
      SnowBallWH         = 16;
      SnowBallDestroyFS  = 0.2;
      SnowBallDestroyFMax= 4;
      RangeSnowBall      = 3;
      CBonusTime         = 800;
      TestBeginImpuls    = 4;
      TestSilaTyajesty   = 0.07;
      AnimSpeed          = 0.1;

type TScore = class
       x,y   : real;
       num   : integer;
       time  : integer;
       active: boolean;
       procedure Init(scoreX,scoreY: real; scoreNum: integer);
       procedure Draw(CameraX: integer);
       procedure Process;
     end;

type TEffect = class
       rect  : Trect;
       frame : real;
       active: boolean;
       gamer : boolean;
       procedure Init(x,y,w,h: integer; ongamer: boolean);
       procedure Draw(CameraX: integer);
       procedure Process;
     end;

type TSnowBall = class
       rect  : Trect;
       angle : integer;
       vector: integer;
       active: boolean;
       procedure Init(x,y,w,h,vectorPath: integer);
       procedure Draw(CameraX: integer);
       procedure Process;
     end;

type TSnowBallDestroy = class
       rect  : Trect;
       Frame : real;
       active: boolean;
       procedure Init(x,y,w,h: integer);
       procedure Draw(CameraX: integer);
       procedure Process;
     end;

type TPlayer = class
       Rect      : TRect;
       Health    : integer;
       Frame     : real;
       FrameHand : real;
       HandX     : integer;
       HandY     : integer;
       Score     : integer;
       Jump      : boolean;
       JumpY     : integer;
       ReloadAmmo: integer;
       Speed     : integer;
       Visible   : boolean;
       TimeNoKill: real;
       Vector    : integer;
       BonusTime : integer;
       Impuls    : real;
       SpeedDown : real;
       procedure Init(x,y: integer; dead: boolean);
       procedure Draw(CameraX: integer);
       procedure IncHealth(numHealth: integer);
       procedure Process;
     end;

procedure HealthEnd;
procedure ScoreCreate(x,y,num: integer);
procedure EffectCreate(x,y,w,h: integer; ongamer: boolean);
procedure SnowBallCreate(x,y,w,h,vectorPath: integer);
procedure SnowBallDestroyCreate(x,y,w,h: integer);
procedure DifferentDraw;
procedure DifferentProcess;

var texPlayer         : cardinal;
    texPlayerHand     : cardinal;
    texEffect         : cardinal;
    texSnowBall       : cardinal;
    texSnowBallDestroy: cardinal;
    Players           : TPlayer;
    PlayerMove        : boolean;
    ShotFlag          : boolean;
    Scores            : array of TScore;
    Effect            : array of TEffect;
    SnowBalls         : array of TSnowBall;
    SnowBallDestroys  : array of TSnowBallDestroy;

implementation

uses Main, GUI;

{ TScore }

procedure TScore.Init(scoreX,scoreY: real; scoreNum: integer);
begin
  x:= scoreX;
  y:= scoreY;
  num:= scoreNum;
  time:= 0;
  active:= true;
end;

procedure TScore.Draw(CameraX: integer);
begin
  if active = true then
    NumberDraw(round(x)-CameraX,round(y),14,32,num,255-time*2);
    //DrawText2D(0,round(x)-CameraX,round(y),inttostr(num),$0000FF,255-time*2);
end;

procedure TScore.Process;
begin
  if active = true then
  begin
    y:= y-0.5;
    time:= time+1;
    if time > ScoreActiveTime then
      active:= false;
  end;
end;

{ TEffect }

procedure TEffect.Draw(CameraX: integer);
begin
  if active = true then
    DrawSprite2D(texEffect,rect.x-CameraX,rect.y,rect.w,rect.h,0,255,16777215,false,rect.w,rect.h,round(frame),4);
end;

procedure TEffect.Init(x, y, w, h: integer; ongamer: boolean);
begin
  rect:= ReturnRect(x,y,w,h);
  frame:= 1;
  active:= true;
  gamer:= ongamer;
end;

procedure TEffect.Process;
begin
  if active = true then
  begin
    frame:= frame+EffectFrameSpeed;
    if frame > 4 then
    begin
      active:= false;
      if gamer then
      begin
        PausePlayer:= false;
        Players.Visible:= true;
      end;
    end;
  end;
end;

{ TSnowBall }

procedure TSnowBall.Init(x, y, w, h, vectorPath: integer);
begin
  rect:= ReturnRect(x,y,w,h);
  angle:= 0;
  active:= true;
  vector:= vectorPath;
end;

procedure TSnowBall.Draw(CameraX: integer);
begin
  if active then
    DrawSprite2D(texSnowBall,rect.x-CameraX,rect.y,rect.w,rect.h,angle);
end;

procedure TSnowBall.Process;
var numElement: integer;
begin
  if active then
  begin
    angle:= angle+SnowBallAngleSpeed*vector;
    rect.x:= rect.x+SnowBallSpeed*vector;
    if (rect.x < 0) or (rect.x > Levels[CurrentLevel].Width) or
       (FlagEndRound = true) or (FlagEndGame = true) then
      active:= false;
    for numElement:= 0 to Levels[CurrentLevel].countGround-1 do
    begin
      if Levels[CurrentLevel].Ground[numElement].Fon = false then
      begin
        if RectInRect(rect,Levels[CurrentLevel].Ground[numElement].Rect) then
        begin
          SnowBallDestroyCreate(rect.x,rect.y,rect.w,rect.h);
          active:= false;
          Break;
        end;
      end;
    end;
    for numElement:= 0 to Levels[CurrentLevel].countTrap-1 do
    begin
      if Levels[CurrentLevel].Trap[numElement].Id = TrapSnowBall then
      begin
        if RectInRect(rect,Levels[CurrentLevel].Trap[numElement].Rect) then
        begin
          SnowBallDestroyCreate(rect.x,rect.y,rect.w,rect.h);
          active:= false;
          Break;
        end;
      end;
    end;
    for numElement:= 0 to Levels[CurrentLevel].countEnemy-1 do
    begin
      if Levels[CurrentLevel].EnemyLevel[numElement].Life = true then
      begin
        if RectInRect(rect,Levels[CurrentLevel].EnemyLevel[numElement].Rect) then
        begin
          SnowBallDestroyCreate(rect.x,rect.y,rect.w,rect.h);
          active:= false;
          Levels[CurrentLevel].EnemyLevel[numElement].Health:=
          Levels[CurrentLevel].EnemyLevel[numElement].Health-1;
          Break;
        end;
      end;
    end;
    if (RectInRect(rect,Players.Rect) = true) and (Players.TimeNoKill = 0) then
    begin
      Players.IncHealth(-1);
      active:= false;
      SnowBallDestroyCreate(rect.x,rect.y,rect.w,rect.h);
    end;
  end;
end;

{ TSnowBallDestroy }

procedure TSnowBallDestroy.Init(x, y, w, h: integer);
begin
  rect:= ReturnRect(x,y,w,h);
  Frame:= 1;
  active:= true;
end;

procedure TSnowBallDestroy.Draw(CameraX: integer);
begin
  if active then
    DrawSprite2D(texSnowBallDestroy,rect.x-CameraX,rect.y,rect.w,rect.h,0,255,$FFFFFF,
                 false,SnowBallWH,SnowBallWH,round(Frame),SnowBallDestroyFMax);
end;

procedure TSnowBallDestroy.Process;
begin
  if active then
  begin
    Frame:= Frame+SnowBallDestroyFS;
    if Frame > SnowBallDestroyFMax+0.4 then
      active:= false;
  end;
end;

{ TPlayer }

procedure TPlayer.Draw(CameraX: integer);
var alfa: integer;
begin
  if TimeNoKill = 0 then
    alfa:= 255
  else
  if TimeNoKill < MaxTimeNoKill div 2 then
  begin
    if round(TimeNoKill) mod 7 = 0 then
      alfa:= 255
    else
      alfa:= 100;
  end
  else
    alfa:= 100;
  if Visible then
  begin
    DrawSprite2D(texPlayer,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,alfa,$FFFFFF,true,PlayerW,PlayerH,Round(Frame),PlayerFrameMax);
    DrawSprite2D(texPlayerHand,HandX,HandY,32,32,0,alfa,$FFFFFF,true,32,32,Round(FrameHand),PlayerHandFrameMax);
  end;
end;

procedure TPlayer.Init(x, y: integer; dead: boolean);
begin
  Rect:= ReturnRect(x,y,PlayerW,PlayerH);
  Health:= PlayerMaxHealth;
  Frame:= 1;
  FrameHand:= 1;
  Jump:= false;
  JumpY:= 0;
  ReloadAmmo:= 0;
  Speed:= PlayerSpeed;
  Visible:= false;
  TimeNoKill:= MaxTimeNoKill;
  Vector:= rigthVector;
  BonusTime:= 0;
  ShotFlag:= false;
  if dead = false then
    Score:= 0;
end;

procedure TPlayer.Process;
var numGround : integer;
    numElement: integer;
    Left,Rigth,
    Up,Down   : boolean;
begin
  Left:= true;
  Rigth:= true;
  Up:= true;
  Down:= true;
  PlayerMove:= false;
  if Vector = rigthVector then
  begin
    HandX:= Rect.x-CameraMainX+(PlayerW div 2)-10;
    HandY:= Rect.y+32;
  end;
  if Vector = leftVector then
  begin
    HandX:= Rect.x-CameraMainX+(PlayerW div 2)-20;
    HandY:= Rect.y+32;
  end;
  for numGround:= 0 to Levels[CurrentLevel].countGround-1 do
  begin
    if Levels[CurrentLevel].Ground[numGround].Fon = true then
      continue;
    if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+Speed,Rect.y-SpeedJump,Rect.w-Speed*2,Rect.h)) then
      Up:= false;
    if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+Speed,Rect.y+2,Rect.w-Speed*2,Rect.h)) then
    begin
      Down:= false;
      SpeedDown:= 0;
      Rect.y:= Levels[CurrentLevel].Ground[numGround].Rect.y-Rect.h;
    end;
    if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x-Speed,Rect.y,Rect.w,Rect.h)) then
    begin
      Left:= false;
      Rect.x:= Levels[CurrentLevel].Ground[numGround].Rect.x+
         Levels[CurrentLevel].Ground[numGround].Rect.w;
    end;
    if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                    ReturnRect(Rect.x+Speed,Rect.y,Rect.w,Rect.h)) then
    begin
      Rigth:= false;
      Rect.x:= Levels[CurrentLevel].Ground[numGround].Rect.x-Rect.w;
    end;
  end;
  if inp.Down(Key_Left)  then
  begin
    if ShotFlag = false then
    begin
      Vector:= leftVector;
      if Frame < 4.6 then
        Frame:= 4.7;
    end;
    PlayerMove:= true;
    if (Left = true) and (Rect.x-Speed > 0) then
      Rect.x:= Rect.x-Speed;
  end;
  if inp.Down(Key_Right) then
  begin
    if ShotFlag = false then
    begin
      Vector:= rigthVector;
      if Frame > 4.6 then
        Frame:= 0.7;
    end;
    PlayerMove:= true;
    if (Rigth = true) and (Rect.x+Rect.w+Speed < Levels[CurrentLevel].Width) then
      Rect.x:= Rect.x+Speed;
  end;
  if (inp.Down(Key_Down) = true) and (Jump = true) then
  begin
    Jump:= false;
    SpeedDown:= 0;
  end;
  if (inp.Down(Key_Space) = true) and (ReloadAmmo = 0) and (ShotFlag = false) then
  begin
    if Vector = leftVector then
      FrameHand:= 10;
    if Vector = rigthVector then
      FrameHand:= 5;
    ShotFlag:= true;
    {if BonusTime > 0 then
      ReloadAmmo:= PlayerMaxReload div 3
    else
      ReloadAmmo:= PlayerMaxReload;
    if Vector = rigthVector then
      SnowBallCreate(Rect.x+Rect.w+RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector)
    else if Vector = leftVector then
      SnowBallCreate(Rect.x-SnowBallWH-RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector);}
  end;
  if (FrameHand >= 4.7) and (frameHand < 9.7) then
  begin
    if FrameHand >= 9.0 then
    begin
      FrameHand:= 0.7;
      ShotFlag:= false;
      if BonusTime > 0 then
        ReloadAmmo:= PlayerMaxReload div 2
      else
        ReloadAmmo:= PlayerMaxReload;
      SnowBallCreate(Rect.x+Rect.w+RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector)
    end
    else
    begin
      if BonusTime > 0 then
        FrameHand:= FrameHand+AnimSpeed*3.5
      else
        FrameHand:= FrameHand+AnimSpeed*2;
      if FrameHand > 4 then
        HandX:= Rect.x-CameraMainX+(PlayerW div 2)-10;
      if FrameHand > 6.5 then
      begin
        HandY:= Rect.y+24;
        HandX:= Rect.x-CameraMainX+(PlayerW div 2)-10
      end;
    end;
  end;
  if (FrameHand >= 9.7) and (frameHand < 14.7) then
  begin
    if FrameHand >= 14.0 then
    begin
      FrameHand:= 0.7;
      ShotFlag:= false;
      if BonusTime > 0 then
        ReloadAmmo:= PlayerMaxReload div 2
      else
        ReloadAmmo:= PlayerMaxReload;
      SnowBallCreate(Rect.x-SnowBallWH-RangeSnowBall,Rect.y+(Rect.h div 2)-SnowBallWH,SnowBallWH,SnowBallWH,Vector);
    end
    else
    begin
      if BonusTime > 0 then
        FrameHand:= FrameHand+AnimSpeed*3.5
      else
        FrameHand:= FrameHand+AnimSpeed*2;
      if FrameHand > 10 then
        HandX:= Rect.x-CameraMainX+(PlayerW div 2)-20;
      if FrameHand > 12.5 then
      begin
        HandY:= Rect.y+24;
        HandX:= Rect.x-CameraMainX+(PlayerW div 2)-20
      end;
    end;
  end;
  if PlayerMove = true then
  begin
    if Vector = rigthVector then
    begin
      if Frame > 4.3 then
        Frame:= 0.7
      else
        Frame:= Frame+AnimSpeed;
    end;
    if Vector = leftVector then
    begin
      if Frame > 8.3 then
        Frame:= 4.7
      else
        Frame:= Frame+AnimSpeed;
    end;
  end;
  if (FrameHand < 4.7) and (PlayerMove = true) then
  begin
    if FrameHand > 4.3 then
      FrameHand:= 0.7
    else
      FrameHand:= FrameHand+AnimSpeed;
  end;
  if Jump = false then
  begin
    if Down then
      Rect.y:= Rect.y+round(SpeedDown)
    else
    begin
      if inp.Down(Key_Up) then
      begin
        Jump:= true;
        JumpY:= Rect.y;
        SpeedDown:= 0;
        Impuls:= TestBeginImpuls;
      end;
    end;
  end;
  {if Jump = true then
  begin
    if Up then
      Rect.y:= Rect.y-SpeedJump
    else
      Jump:= false;
    if Rect.y < JumpY-JumpWidth then
      Jump:= false;
  end;}
  if Jump = true then
  begin
    Impuls:= Impuls-TestSilaTyajesty;
    if Up then
      Rect.y:= Rect.y-round(impuls)
    else
    begin
      Jump:= false;
      SpeedDown:= 0;
    end;
    {if Rect.y < JumpY-JumpWidth then
      Jump:= false;}
    if Impuls < 0 then
    begin
      Jump:= false;
      SpeedDown:= 0;
    end;
  end;
  if Down then
  begin
    if SpeedDown < SpeedDownMax then
      SpeedDown:= SpeedDown+TestSilaTyajesty*2;
  end;
  for numElement:= 0 to Length(Levels[CurrentLevel].Bonus)-1 do
  begin
    if Levels[CurrentLevel].Bonus[numElement].Enable = true then
    begin
      if RectInRect(Rect,Levels[CurrentLevel].Bonus[numElement].Rect) then
      begin
        if Levels[CurrentLevel].Bonus[numElement].Id = BonusConfeta then
          if Health < 3 then
            IncHealth(1);
        if Levels[CurrentLevel].Bonus[numElement].Id = BonusOreh then
          TimeNoKill:= MaxTimeNoKill*3;
        if Levels[CurrentLevel].Bonus[numElement].Id = BonusPodarok then
          BonusTime:= CBonusTime;
        ScoreCreate(Levels[CurrentLevel].Bonus[numElement].Rect.x,
                    Levels[CurrentLevel].Bonus[numElement].Rect.y,
                    BonusScore(Levels[CurrentLevel].Bonus[numElement].Id));
        EffectCreate(Levels[CurrentLevel].Bonus[numElement].Rect.x,
                     Levels[CurrentLevel].Bonus[numElement].Rect.y,
                     Levels[CurrentLevel].Bonus[numElement].Rect.w,
                     Levels[CurrentLevel].Bonus[numElement].Rect.h,false);
        Score:= Score+BonusScore(Levels[CurrentLevel].Bonus[numElement].Id);
        Levels[CurrentLevel].Bonus[numElement].Enable:= false;
      end;
    end;
  end;
  for numElement:= 0 to Length(Levels[CurrentLevel].Place)-1 do
  begin
    if RectInRect(Rect,Levels[CurrentLevel].Place[numElement].Rect) then
    begin
      case Levels[CurrentLevel].Place[numElement].ID of
        PlaceSave    : begin
                         if Levels[CurrentLevel].Place[numElement].Check = false then
                         begin
                           Levels[CurrentLevel].Place[numElement].Check:= true;
                           EffectCreate(
                           Levels[CurrentLevel].Place[numElement].Rect.x,
                           Levels[CurrentLevel].Place[numElement].Rect.y,
                           Levels[CurrentLevel].Place[numElement].Rect.w,
                           Levels[CurrentLevel].Place[numElement].Rect.h,false
                                        );
                         end;
                       end;
        PlaceEndRound: begin
                         LevelEnd(false);
                         EffectCreate(
                         Levels[CurrentLevel].Place[numElement].Rect.x,
                         Levels[CurrentLevel].Place[numElement].Rect.y,
                         Levels[CurrentLevel].Place[numElement].Rect.w,
                         Levels[CurrentLevel].Place[numElement].Rect.h,false
                                      );
                         Levels[CurrentLevel].Place[numElement].Enable:= false;
                       end;
        PlaceEndGame : begin
                         LevelEnd(true);
                         EffectCreate(
                         Levels[CurrentLevel].Place[numElement].Rect.x,
                         Levels[CurrentLevel].Place[numElement].Rect.y,
                         Levels[CurrentLevel].Place[numElement].Rect.w,
                         Levels[CurrentLevel].Place[numElement].Rect.h,false
                                      );
                         Levels[CurrentLevel].Place[numElement].Enable:= false;
                       end;
      end;
    end;
  end;
  if (Health <= 0) or (Rect.y > wnd.Height) then
  begin
    HealthEnd;
  end;
  if (CameraMainX > 0) and (Rect.x < CameraMainX+ScreenMoveLimit) then
  begin
    CameraMainX:= CameraMainX-CameraSpeed;
  end;
  if (CameraMainX < Levels[CurrentLevel].Width-wnd.Width) and (Rect.x > CameraMainX+wnd.Width-ScreenMoveLimit) then
  begin
    CameraMainX:= CameraMainX+CameraSpeed;
  end;
  if TimeNoKill < 0 then
    TimeNoKill:= 0
  else if TimeNoKill > 0 then
    TimeNoKill:= TimeNoKill-1;
  if BonusTime < 0 then
    BonusTime:= 0
  else
    BonusTime:= BonusTime-1;
  if ReloadAmmo < 0 then
    ReloadAmmo:= 0
  else if ReloadAmmo > 0 then
    ReloadAmmo:= ReloadAmmo-1;
end;

procedure TPlayer.IncHealth(numHealth: integer);
begin
  if numHealth < 0 then
    TimeNoKill:= MaxTimeNoKill;
  Health:= Health+numHealth;
end;

procedure HealthEnd;
var numElement: integer;
    tmpX, tmpY: integer;
    tmpStartX,
    tmpStartY : integer;
begin
  tmpX:= DefaultStartX;
  tmpY:= DefaultStartY;
  tmpStartX:= DefaultStartX;
  tmpStartY:= DefaultStartY;
  for numElement:= 0 to levels[CurrentLevel].countPlace-1 do
  begin
    if levels[CurrentLevel].Place[numElement].ID = PlaceStart then
    begin
      tmpStartX:= levels[CurrentLevel].Place[numElement].Rect.x;
      tmpStartY:= levels[CurrentLevel].Place[numElement].Rect.y;
    end;
    if (levels[CurrentLevel].Place[numElement].ID = PlaceSave) and
       (levels[CurrentLevel].Place[numElement].Check = true) then
    begin
      if levels[CurrentLevel].Place[numElement].Rect.x > tmpX then
      begin
        tmpX:= levels[CurrentLevel].Place[numElement].Rect.x;
        tmpY:= levels[CurrentLevel].Place[numElement].Rect.y;
      end;
    end;
    if (tmpX = DefaultStartX) and (tmpY = DefaultStartY) then
    begin
      tmpX:= tmpStartX;
      tmpY:= tmpStartY;
    end;
  end;
  EffectCreate(tmpX,tmpY-PlayerH div 2,PlayerW,PlayerH,true);
  CameraMainX:= tmpX-wnd.Width div 2;
  if CameraMainX < 0 then
    CameraMainX:= 0;
  if CameraMainX+wnd.Width > Levels[CurrentLevel].Width then
    CameraMainX:= Levels[CurrentLevel].Width-wnd.Width;
  {Players.Rect.x:= tmpX;
  Players.Rect.y:= tmpY-PlayerH div 2;
  Players.Jump:= false;
  Players.JumpY:= Players.Rect.y;
  PausePlayer:= true;
  Players.Visible:= false;
  Players.Health:= PlayerMaxHealth; }
  PausePlayer:= true;
  Players.Init(tmpX,tmpY-PlayerH div 2,true);
end;

procedure ScoreCreate(x,y,num: integer);
var i: integer;
begin
  for i:= 0 to length(Scores)-1 do
  begin
    if Scores[i].active = false then
    begin
      Scores[i].Init(x,y,num);
      exit;
    end;
  end;
  SetLength(Scores,Length(Scores)+1);
  Scores[Length(Scores)-1]:= TScore.Create;
  Scores[Length(Scores)-1].Init(x,y,num);
end;

procedure EffectCreate(x,y,w,h: integer; ongamer: boolean);
var i: integer;
begin
  for i:= 0 to length(Effect)-1 do
  begin
    if Effect[i].active = false then
    begin
      Effect[i].Init(x,y,w,h,ongamer);
      exit;
    end;
  end;
  SetLength(Effect,Length(Effect)+1);
  Effect[Length(Effect)-1]:= TEffect.Create;
  Effect[Length(Effect)-1].Init(x,y,w,h,ongamer);
end;

procedure SnowBallCreate(x,y,w,h,vectorPath: integer);
var i: integer;
begin
  for i:= 0 to length(SnowBalls)-1 do
  begin
    if SnowBalls[i].active = false then
    begin
      SnowBalls[i].Init(x,y,w,h,vectorPath);
      exit;
    end;
  end;
  SetLength(SnowBalls,Length(SnowBalls)+1);
  SnowBalls[Length(SnowBalls)-1]:= TSnowBall.Create;
  SnowBalls[Length(SnowBalls)-1].Init(x,y,w,h,vectorPath);
end;

procedure SnowBallDestroyCreate(x,y,w,h: integer);
var i: integer;
begin
  for i:= 0 to length(SnowBallDestroys)-1 do
  begin
    if SnowBallDestroys[i].active = false then
    begin
      SnowBallDestroys[i].Init(x,y,w,h);
      exit;
    end;
  end;
  SetLength(SnowBallDestroys,Length(SnowBallDestroys)+1);
  SnowBallDestroys[Length(SnowBallDestroys)-1]:= TSnowBallDestroy.Create;
  SnowBallDestroys[Length(SnowBallDestroys)-1].Init(x,y,w,h);
end;

procedure DifferentDraw;
var i: integer;
begin
  for i:= 0 to Length(Effect)-1 do
  begin
    Effect[i].Draw(CameraMainX);
  end;
  for i:= 0 to Length(Scores)-1 do
  begin
    Scores[i].Draw(CameraMainX);
  end;
  for i:= 0 to Length(SnowBalls)-1 do
  begin
    SnowBalls[i].Draw(CameraMainX);
  end;
  for i:= 0 to Length(SnowBallDestroys)-1 do
  begin
    SnowBallDestroys[i].Draw(CameraMainX);
  end;
end;

procedure DifferentProcess;
var i: integer;
begin
  for i:= 0 to Length(Effect)-1 do
  begin
    Effect[i].Process;
  end;
  for i:= 0 to Length(Scores)-1 do
  begin
    Scores[i].Process;
  end;
  for i:= 0 to Length(SnowBalls)-1 do
  begin
    SnowBalls[i].Process;
  end;
  for i:= 0 to Length(SnowBallDestroys)-1 do
  begin
    SnowBallDestroys[i].Process;
  end;
end;

end.
