unit SpriteEngine;

interface

uses
  OpenGL,
  Windows,
  SysUtils,
  eXgine,
  Consts;

procedure DrawSprite2D_simple(tex_sprite: cardinal;x,y,w,h: integer; alfa: integer = 255;
          color: integer = $FFFFFF; alfa_enable: boolean = false);
procedure DrawSprite2D(tex_sprite: cardinal;x,y,w,h: integer;
          Angle: integer = 0; alfa: integer = 255; color: integer = $FFFFFF; alfa_enable: boolean = false;
          FrameWidth: integer = 0; FrameHeight: integer = 0;
          Frame: byte = 1; FrameCountX: byte = 1);
procedure DrawText2D(font: cardinal; x,y: integer; text: string; color: integer = $FFFFFF; alfa: integer = 255);
procedure DrawLine2D(x,y,x1,y1: integer; color: integer = $FFFFFF; alfa: integer = 255;width: real = 1.0);

implementation

procedure DrawSprite2D_simple(tex_sprite: cardinal;x,y,w,h: integer; alfa: integer = 255;
          color: integer = $FFFFFF; alfa_enable: boolean = false);
begin
  tex.Enable(tex_sprite);
  ogl.Blend(1);
  glPushMatrix;
  glTranslatef(x,y,0);
  if alfa_enable = true then
  begin
    glEnable(GL_ALPHA_TEST);       //   
    glAlphaFunc(GL_GEQUAL, 0);
  end;
  glColor4ub(GetRValue(Color),GetGValue(Color),GetBValue(Color),alfa);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2f(0, 0);
    glTexCoord2f(1, 0); glVertex2f(w, 0);
    glTexCoord2f(1, 1); glVertex2f(w, h);
    glTexCoord2f(0, 1); glVertex2f(0, h);
  glEnd;
  if alfa_enable = true then
  begin
    glDisable(GL_ALPHA_TEST);
  end;
  glPopMatrix;
  tex.Disable;
end;

procedure DrawSprite2D(tex_sprite: cardinal;x,y,w,h: integer;
          Angle: integer = 0; alfa: integer = 255; color: integer = $FFFFFF; alfa_enable: boolean = false;
          FrameWidth: integer = 0; FrameHeight: integer = 0;
          Frame: byte = 1; FrameCountX: byte = 1);
var Sprite: boolean;
    imgWidth: glfloat;
    FrameX: integer;
begin
  tex.Enable(tex_sprite);
  ogl.Blend(1);
  glPushMatrix;
  if Frame+FrameCountX = 2 then
  begin
    Sprite:= false;
    GlTranslatef(x+(w div 2),y+(h div 2),0);
  end
  else
  begin
    Sprite:= true;
    GlTranslatef(x+(FrameWidth div 2),y+(FrameHeight div 2),0);
  end;
  if angle<>0 then  glrotatef(Angle,0,0,1);
  if alfa_enable = true then
  begin
    glEnable(GL_ALPHA_TEST);       //   
    glAlphaFunc(GL_GEQUAL, 0);
  end;
  glColor4ub(GetRValue(Color),GetGValue(Color),GetBValue(Color),alfa);
  if Sprite = true then
  begin
    imgWidth:= 1.0/FrameCountX;
    FrameX:= Frame-1;
    glBegin(GL_QUADS);
      glTexCoord2f(imgWidth*FrameX, 0);
      glVertex2f(- FrameWidth div 2, - FrameHeight div 2);
      glTexCoord2f(imgWidth*FrameX+imgWidth, 0);
      glVertex2f(FrameWidth div 2, - FrameHeight div 2);
      glTexCoord2f(imgWidth*FrameX+imgWidth, 1);
      glVertex2f(FrameWidth div 2, FrameHeight div 2);
      glTexCoord2f(imgWidth*FrameX, 1);
      glVertex2f(- FrameWidth div 2, FrameHeight div 2);
    glEnd;
  end
  else
  begin
    glBegin(GL_QUADS);
      glTexCoord2f(0, 0); glVertex2f(- w div 2, - h div 2);
      glTexCoord2f(1, 0); glVertex2f(w div 2, -h div 2);
      glTexCoord2f(1, 1); glVertex2f(w div 2, h div 2);
      glTexCoord2f(0, 1); glVertex2f(-w div 2, h div 2);
    glEnd;
  end;
  if alfa_enable = true then
  begin
    glDisable(GL_ALPHA_TEST);
  end;
  glPopMatrix;
  tex.Disable;
end;

procedure DrawText2D(font: cardinal; x,y: integer; text: string; color: integer = $FFFFFF; alfa: integer = 255);
begin
  glColor4ub(GetRValue(Color),GetGValue(Color),GetBValue(Color),alfa);
  ogl.TextOut(font, x, y, pchar(text));
end;

procedure DrawLine2D(x,y,x1,y1: integer; color: integer = $FFFFFF; alfa: integer = 255;width: real = 1.0);
begin
  glColor4ub(GetRValue(Color),GetGValue(Color),GetBValue(Color),alfa);
  glLineWidth(Width);
  glBegin(GL_LINES);
    glVertex2f(x,y);
    glVertex2f(x1,y1);
  glend;
end;

end.
