unit level;

interface

uses eXgine, SysUtils, Math, MyInterface, EngineUtils, Consts, SpriteEngine, OpenGL, Enemy;

const numTexGround = 4;
      numTexFonObj = 2;
      numTexTrap   = 2;
      groundW      = 64;
      groundH      = 64;
      AngleSpeed   = 1;
      FrameSpeed   = 0.01;
      TrapIceBlade = 1;
      TrapSnowBall = 2;
      IceBladeCountFrame = 3;
      AllFrameTrap = 4;
      BonusCitrus  = 1;
      BonusPodarok = 2;
      BonusConfeta = 3;
      BonusOreh    = 4;
      MaxBonusCount= 4;
      BonusW       = 32;
      BonusH       = 32;
      PlaceStart   = 1;
      PlaceSave    = 2;
      PlaceEndRound= 3;
      PlaceEndGame = 4;
      PlaceW       = 32;
      PlaceH       = 32;
      PlaceCount   = 4;
      MaxLevel     = 3;
      SpeedDownTrap= 2;

type TGround = class
       Rect       : TRect;
       numTex     : integer;
       Active     : boolean;
       Fon        : boolean;
       Speed      : integer;
       StartX     : integer;
       StartY     : integer;
       DeltaXRigth: integer;
       DeltaYUp   : integer;
       Vector     : integer;
       procedure Init(x,y,w,h: integer; numT: integer; FonObject: boolean; SpeedMove,DXR,DYU: integer);
       procedure Process(flagSelect: integer; AnimationFlag: boolean);
       procedure Draw(CameraX: integer);
     end;

type TTrap = class
       Rect      : TRect;
       Id        : integer;
       Active    : boolean;
       Angle     : integer;
       SpeedX    : integer;
       FrameNum  : real;
       procedure Init(x,y,w,h,AngleT,SpeedXT: integer; idTrap: byte);
       procedure Process(flagSelect: integer; Redo: boolean);
       procedure Draw(CameraX: integer);
     end;

type TBonus = class
       Rect      : TRect;
       Id        : integer;
       Active    : boolean;
       Angle     : integer;
       Enable    : boolean;
       procedure Init(x,y,w,h,AngleBonus: integer; idBonus: byte);
       procedure Process(flagSelect: integer);
       procedure Draw(CameraX: integer);
     end;

type TPlace = class
       Rect  : TRect;
       ID    : integer;
       Check : boolean;
       Active: boolean;
       Enable: boolean;
       procedure Init(x,y,w,h: integer; idPlace: byte);
       procedure Process(flagSelect: integer);
       procedure Draw(CameraX: integer);
     end;

type TLevel = class
       Width       : integer;
       countGround : integer;
       Ground      : array of TGround;//   + 
       countTrap   : integer;
       Trap        : array of TTrap;//
       countBonus  : integer;
       Bonus       : array of TBonus;//
       countPlace  : integer;
       Place       : array of TPlace;// , ,  
       countEnemy  : integer;
       EnemyLevel  : array of TEnemy;//
       procedure Init;
       procedure Process(flagSelect: integer; AnimationFlag, Redo  : boolean);
       procedure Draw;
       procedure GroundAdd(x,y,w,h: integer; numT: integer; FonObject: boolean; SpeedMove,DXR,DYU: integer);
       procedure TrapAdd(x,y,w,h,AngleT,SpeedXT: integer; idTrap: byte);
       procedure BonusAdd(x,y,w,h,AngleBonus: integer; idBonus: byte);
       procedure PlaceAdd(x,y,w,h: integer; idPlace: byte);
       procedure EnemyAdd(x,y,w,h: integer; idEnemy: byte);
       procedure AllDelete;
       procedure GroundDelete(num: integer);
       procedure TrapDelete(num: integer);
       procedure BonusDelete(num: integer);
       procedure PlaceDelete(num: integer);
       procedure EnemyDelete(num: integer);
     end;

procedure LevelTexInit;
function  BonusScore(idBonus: integer): integer;
procedure LoadLevel(numLevel: integer);

var texAllGround : cardinal;
    texFonObject : cardinal;
    texTrap      : cardinal;
    texBonus     : cardinal;
    texPlace     : cardinal;
    CurrentLevel : integer;
    Levels       : array[1..MaxLevel] of TLevel;
    PlayerMove   : boolean;

implementation

uses Main, Player, GUI;

procedure DrawSelect(Rect: TRect; CameraX: integer);
begin
  DrawLine2D(Rect.x-CameraX,Rect.y,Rect.x+Rect.w-CameraX,Rect.y,$0000FF);
  DrawLine2D(Rect.x+Rect.w-CameraX,Rect.y,Rect.x+Rect.w-CameraX,Rect.y+Rect.h,$0000FF);
  DrawLine2D(Rect.x+Rect.w-CameraX,Rect.y+Rect.h,Rect.x-CameraX,Rect.y+Rect.h,$0000FF);
  DrawLine2D(Rect.x-CameraX,Rect.y+Rect.h,Rect.x-CameraX,Rect.y,$0000FF);
end;

procedure LevelTexInit;
var numTex: integer;
begin
  texAllGround:= tex.Load('data\ground.tga');
  texFonObject:= tex.Load('data\fonobject.tga');
  texTrap:= tex.Load('data\trap.tga');
  texBonus:= tex.Load('data\bonus.tga');
  texPlace:= tex.Load('data\place.tga');
  for numTex:= 1 to EnemyMaxCount do
    texEnemy[numTex]:= tex.Load(PChar('data\enemy'+inttostr(numTex)+'.tga'));
end;

function  BonusScore(idBonus: integer): integer;
var tmpResult: integer;
begin
  tmpResult:= 0;
  case idBonus of
    BonusCitrus: tmpResult:= 150;
    BonusPodarok: tmpResult:= 500;
    BonusConfeta: tmpResult:= 100;
    BonusOreh: tmpResult:= 80;
  end;
  BonusScore:= tmpResult;
end;

{ TLevel }

procedure TLevel.BonusAdd(x, y, w, h, AngleBonus: integer; idBonus: byte);
begin
  countBonus:= Length(Bonus)+1;
  SetLength(Bonus,countBonus);
  Bonus[countBonus-1]:= TBonus.Create;
  Bonus[countBonus-1].Init(x,y,w,h,AngleBonus,idBonus);
end;

procedure TLevel.EnemyAdd(x,y,w,h: integer; idEnemy: byte);
begin
  countEnemy:= Length(EnemyLevel)+1;
  SetLength(EnemyLevel,countEnemy);
  EnemyLevel[countEnemy-1]:= TEnemy.Create;
  EnemyLevel[countEnemy-1].Init(x,y,w,h,idEnemy);
end;

procedure TLevel.PlaceAdd(x, y, w, h: integer; idPlace: byte);
begin
  countPlace:= Length(Place)+1;
  SetLength(Place,countPlace);
  Place[countPlace-1]:= TPlace.Create;
  Place[countPlace-1].Init(x,y,w,h,idPlace);
end;

procedure TLevel.TrapAdd(x, y, w, h, AngleT, SpeedXT: integer;
  idTrap: byte);
var i: integer;
begin
  for i:= 0 to Length(Ground)-1 do
  begin
    if (RectInRect(ReturnRect(x,y,w,h),Ground[i].Rect) = true) and (Ground[i].Fon = false) then
      exit;
  end;
  countTrap:= Length(Trap)+1;
  SetLength(Trap,countTrap);
  Trap[countTrap-1]:= TTrap.Create;
  Trap[countTrap-1].Init(x,y,w,h,AngleT,SpeedXT,idTrap);
end;

procedure TLevel.GroundAdd(x,y,w,h: integer; numT: integer; FonObject: boolean;
                           SpeedMove,DXR,DYU: integer);
var i: integer;
begin
  for i:= 0 to Length(Ground)-1 do
  begin
    if (RectInRect(ReturnRect(x,y,w,h),Ground[i].Rect) = true) and (FonObject = false)
    and (Ground[i].Fon = false) then
      exit;
  end;
  countGround:= Length(Ground)+1;
  SetLength(Ground,countGround);
  Ground[countGround-1]:= TGround.Create;
  Ground[countGround-1].Init(x,y,w,h,numT,FonObject,SpeedMove,DXR,DYU);
end;

procedure TLevel.AllDelete;
var numElement   : integer;
    flagStop     : boolean;
begin
  flagStop:= False;
  numElement:= 0;
  while not flagStop do
  begin
    if countGround > 0 then
    begin
      if Ground[numElement].Active = true then
      begin
        GroundDelete(numElement);
        flagStop:= false;
      end
      else
      begin
        inc(numElement);
      end;
      if numElement = countGround then
        flagStop:= true;
    end
    else
    begin
      flagStop:= true;
    end;
  end;
  flagStop:= False;
  numElement:= 0;
  while not flagStop do
  begin
    if countTrap > 0 then
    begin
      if Trap[numElement].Active = true then
      begin
        TrapDelete(numElement);
        flagStop:= false;
      end
      else
      begin
        inc(numElement);
      end;
      if numElement = countTrap then
        flagStop:= true;
    end
    else
    begin
      flagStop:= true;
    end;
  end;
  flagStop:= False;
  numElement:= 0;
  while not flagStop do
  begin
    if countBonus > 0 then
    begin
      if Bonus[numElement].Active = true then
      begin
        BonusDelete(numElement);
        flagStop:= false;
      end
      else
      begin
        inc(numElement);
      end;
      if numElement = countBonus then
        flagStop:= true;
    end
    else
    begin
      flagStop:= true;
    end;
  end;
  flagStop:= False;
  numElement:= 0;
  while not flagStop do
  begin
    if countPlace > 0 then
    begin
      if Place[numElement].Active = true then
      begin
        PlaceDelete(numElement);
        flagStop:= false;
      end
      else
      begin
        inc(numElement);
      end;
      if numElement = countPlace then
        flagStop:= true;
    end
    else
    begin
      flagStop:= true;
    end;
  end;
  flagStop:= False;
  numElement:= 0;
  while not flagStop do
  begin
    if countEnemy > 0 then
    begin
      if EnemyLevel[numElement].Active = true then
      begin
        EnemyDelete(numElement);
        flagStop:= false;
      end
      else
      begin
        inc(numElement);
      end;
      if numElement = countEnemy then
        flagStop:= true;
    end
    else
    begin
      flagStop:= true;
    end;
  end;
end;

procedure TLevel.BonusDelete(num: integer);
var i: integer;
begin
  for i:= num to countBonus-2 do
  begin
    Bonus[i]:= Bonus[i+1];
  end;
  SetLength(Bonus,Length(Bonus)-1);
  countBonus:= countBonus-1;
end;

procedure TLevel.EnemyDelete(num: integer);
var i: integer;
begin
  for i:= num to countEnemy-2 do
  begin
    EnemyLevel[i]:= EnemyLevel[i+1];
  end;
  SetLength(EnemyLevel,Length(EnemyLevel)-1);
  countEnemy:= countEnemy-1;
end;

procedure TLevel.GroundDelete(num: integer);
var i: integer;
begin
  for i:= num to countGround-2 do
  begin
    Ground[i]:= Ground[i+1];
  end;
  SetLength(Ground,Length(Ground)-1);
  countGround:= countGround-1;
end;

procedure TLevel.PlaceDelete(num: integer);
var i: integer;
begin
  for i:= num to countPlace-2 do
  begin
    Place[i]:= Place[i+1];
  end;
  SetLength(Place,Length(Place)-1);
  countPlace:= countPlace-1;
end;

procedure TLevel.TrapDelete(num: integer);
var i: integer;
begin
  for i:= num to countTrap-2 do
  begin
    Trap[i]:= Trap[i+1];
  end;
  SetLength(Trap,Length(Trap)-1);
  countTrap:= countTrap-1;
end;

procedure TLevel.Init;
begin
  countGround:= 0;
  SetLength(Ground,countGround);
  countTrap:= 0;
  SetLength(Trap,countTrap);
  countBonus:= 0;
  SetLength(Bonus,countBonus);
  countPlace:= 0;
  SetLength(Place,countPlace);
  countEnemy:= 0;
  SetLength(EnemyLevel,countEnemy);
  Width:= 3840;
end;

procedure TLevel.Draw;
var countElement: integer;
begin
  for countElement:= 0 to countGround-1 do
  begin
    Ground[countElement].Draw(CameraMainX);
  end;
  for countElement:= 0 to countBonus-1 do
  begin
    Bonus[countElement].Draw(CameraMainX);
  end;
  for countElement:= 0 to countPlace-1 do
  begin
    Place[countElement].Draw(CameraMainX);
  end;
  for countElement:= 0 to countTrap-1 do
  begin
    Trap[countElement].Draw(CameraMainX);
  end;
  for countElement:= 0 to countEnemy-1 do
  begin
    EnemyLevel[countElement].Draw(CameraMainX);
  end;
end;

procedure TLevel.Process(flagSelect: integer; AnimationFlag, Redo: boolean);
var countElement: integer;
begin
  PlayerMove:= false;
  for countElement:= 0 to countGround-1 do
  begin
    Ground[countElement].Process(flagSelect,AnimationFlag);
  end;
  for countElement:= 0 to countTrap-1 do
  begin
    Trap[countElement].Process(flagSelect,redo);
  end;
  for countElement:= 0 to countBonus-1 do
  begin
    Bonus[countElement].Process(flagSelect);
  end;
  for countElement:= 0 to countPlace-1 do
  begin
    Place[countElement].Process(flagSelect);
  end;
  for countElement:= 0 to countEnemy-1 do
  begin
    EnemyLevel[countElement].Process(flagSelect,Redo);
  end;
end;

{ TGround }

procedure TGround.Draw(CameraX: integer);
begin
  if Fon = false then
  begin
    DrawSprite2D(texAllGround,Rect.x-CameraX,rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,numTex,numTexGround);
  end
  else
  begin
    DrawSprite2D(texFonObject,Rect.x-CameraX,rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,numTex,numTexFonObj);
  end;
  if Active = true then
    DrawSelect(Rect,CameraX);
end;

procedure TGround.Init(x, y, w, h: integer; numT: integer; FonObject: boolean; SpeedMove,DXR,DYU: integer);
begin
  Rect:= ReturnRect(x,y,w,h);
  numTex:= numT;
  Active:= False;
  Fon:= FonObject;
  Speed:= SpeedMove;
  StartX:= x;
  StartY:= y;
  DeltaXRigth:= DXR;
  DeltaYUp:= DYU;
  Vector:= 1;
end;

procedure TGround.Process(flagSelect: integer; AnimationFlag: boolean);
begin
  if AnimationFlag = true then
  begin
    if Speed <> 0 then
    begin
      if DeltaXRigth <> 0 then
      begin
        if (Rect.x < StartX+DeltaXRigth) and (Vector = 1) then
        begin
          {if (RectInRect(ReturnRect(Rect.x,Rect.y-1,Rect.w, rect.h),Players.Rect) = true)
             and (PlayerMove = false) then
          begin
            Players.Rect.x:= Players.Rect.x+Speed;
            PlayerMove:= true;
          end;}
          Rect.x:= Rect.x+Speed;
          if Rect.x >= StartX+DeltaXRigth then
            Vector:= -1;
        end
        else
        begin
          {if (RectInRect(ReturnRect(Rect.x,Rect.y-1,Rect.w, rect.h),Players.Rect) = true)
             and (PlayerMove = false) then
          begin
            Players.Rect.x:= Players.Rect.x-Speed;
            PlayerMove:= true;
          end;}
          Rect.x:= Rect.x-Speed;
          if Rect.x <= StartX then
            Vector:= 1;
        end;
      end;
      if DeltaYUp <> 0 then
      begin
        if (Rect.y < StartY+DeltaYUp) and (Vector = 1) then
        begin
          Rect.y:= Rect.y+Speed;
          if Rect.y >= StartY+DeltaYUp then
            Vector:= -1;
        end
        else
        begin
          Rect.y:= Rect.y-Speed;
          if Rect.y <= StartY then
            Vector:= 1;
        end;
      end;
    end;
  end;
  if flagSelect <> 0 then
    exit;
  if PointInRect(CursorX+CameraMainX,CursorY,Rect) then
  begin
    if (GetMouseButtonPressed = MB_LEFT) and (MouseDown = MB_NONE) then
    begin
      if Active = true then
        Active:= false
      else
        Active:= true;
    end;
  end;
end;

{ TTrap }

procedure TTrap.Draw(CameraX: integer);
begin
  if Id = TrapIceBlade then
  begin
    DrawSprite2D(texTrap,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(FrameNum),AllFrameTrap);
  end
  else if Id = TrapSnowBall then
  begin
    DrawSprite2D(texTrap,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(FrameNum),AllFrameTrap);
  end;
  if Active = true then
    DrawSelect(Rect,CameraX);
end;

procedure TTrap.Init(x, y, w, h, AngleT, SpeedXT: integer; idTrap: byte);
begin
  Rect:= ReturnRect(x,y,w,h);
  Angle:= AngleT;
  Id:= idTrap;
  Active:= false;
  SpeedX:= SpeedXT;
  if Id = TrapIceBlade then
    FrameNum:= 1
  else
    FrameNum:= AllFrameTrap;
end;

procedure TTrap.Process(flagSelect: integer; Redo: boolean);
var FlagGround: boolean;
    numGround : integer;
    FlagX     : boolean;
begin
  if Redo = false then
  begin
    if Id = TrapIceBlade then
    begin
      if FrameNum > IceBladeCountFrame+0.3 then
        FrameNum:= 0.6
      else
        FrameNum:= FrameNum+FrameSpeed;
      if round(FrameNum) = 2 then
      begin
        if (RectInRect(ReturnRect(Rect.x,Rect.y+(Rect.h div 2),Rect.w, rect.h div 2),
            Players.Rect) = true) and (Players.TimeNoKill = 0) then
          begin
            Players.IncHealth(-1);
          end;
      end;
      if round(FrameNum) = 3 then
      begin
        if (RectInRect(Rect,Players.Rect) = true) and (Players.TimeNoKill = 0) then
        begin
          Players.IncHealth(-1);
        end;
      end;
    end
    else if Id = TrapSnowBall then
    begin
      if Angle > 0 then
        Angle:= Angle+AngleSpeed*SpeedX
      else
        Angle:= 360;
      FlagGround:= false;
      for numGround:= 0 to Levels[CurrentLevel].countGround-1 do
      begin
        if (RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                      ReturnRect(Rect.x,Rect.y+1,Rect.w,Rect.h)) = true)
           and (Levels[CurrentLevel].Ground[numGround].Fon = false) then
        begin
          Rect.y:= Levels[CurrentLevel].Ground[numGround].Rect.y-Rect.h;
          FlagGround:= true;
          Break;
        end;
      end;
      if FlagGround = false then
      begin
        Rect.y:= Rect.y+SpeedDownTrap;
        SpeedX:= 0;
      end
      else
      begin
        FlagX:= false;
        if SpeedX = 0 then
          SpeedX:= -1;
        for numGround:= 0 to Levels[CurrentLevel].countGround-1 do
        begin
          if Levels[CurrentLevel].Ground[numGround].Fon = true then
            Continue;
          if RectInRect(Levels[CurrentLevel].Ground[numGround].Rect,
                        ReturnRect(Rect.x+SpeedX,Rect.y,Rect.w,Rect.h)) then
          begin
            FlagX:= true;
            Break;
          end;
        end;
        if FlagX = true then
          SpeedX:= SpeedX*-1;
      end;
      Rect.x:= Rect.x+SpeedX;
      if (RectInRect(Rect,Players.Rect) = true) and (Players.TimeNoKill = 0) then
      begin
        Players.IncHealth(-1);
      end;
    end;
  end;
  if flagSelect <> 0 then
    exit;
  if PointInRect(CursorX+CameraMainX,CursorY,Rect) then
  begin
    if (GetMouseButtonPressed = MB_LEFT) and (MouseDown = MB_NONE) then
    begin
      if Active = true then
        Active:= false
      else
        Active:= true;
    end;
  end;
end;

{ TBonus }

procedure TBonus.Init(x, y, w, h, AngleBonus: integer; idBonus: byte);
begin
  Rect:= ReturnRect(x,y,w,h);
  Angle:= AngleBonus;
  Id:= idBonus;
  Enable:= true;
  Active:= false;
end;

procedure TBonus.Draw(CameraX: integer);
begin
  if Enable = false then
    exit;
  case Id of
    BonusCitrus:
      DrawSprite2D(texBonus,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(BonusCitrus),MaxBonusCount);
    BonusPodarok:
      DrawSprite2D(texBonus,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(BonusPodarok),MaxBonusCount);
    BonusConfeta:
      DrawSprite2D(texBonus,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(BonusConfeta),MaxBonusCount);
    BonusOreh:
      DrawSprite2D(texBonus,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,Angle,255,$FFFFFF,true,Rect.w,Rect.h,round(BonusOreh),MaxBonusCount);
  end;
  if Active = true then
    DrawSelect(Rect,CameraX);
end;

procedure TBonus.Process(flagSelect: integer);
begin
  if Enable = false then
    exit;
  if flagSelect <> 0 then
    exit;
  if PointInRect(CursorX+CameraMainX,CursorY,Rect) then
  begin
    if (GetMouseButtonPressed = MB_LEFT) and (MouseDown = MB_NONE) then
    begin
      if Active = true then
        Active:= false
      else
        Active:= true;
    end;
  end;
end;

{ TPlace }

procedure TPlace.Draw(CameraX: integer);
begin
  if Enable = false then
    exit;
  case Id of
    PlaceStart:
      DrawSprite2D(texPlace,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,PlaceStart,PlaceCount);
    PlaceSave:
      begin
        DrawSprite2D(texPlace,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,PlaceSave,PlaceCount);
        if Check = true then
          DrawSprite2D_simple(texSnow1,Rect.x+8-CameraX,Rect.y-12,16,16,255,$0000FF,false);
      end;
    PlaceEndRound:
      DrawSprite2D(texPlace,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,PlaceEndRound,PlaceCount);
    PlaceEndGame:
      DrawSprite2D(texPlace,Rect.x-CameraX,Rect.y,Rect.w,Rect.h,0,255,$FFFFFF,true,Rect.w,Rect.h,PlaceEndGame,PlaceCount);
  end;
  if Active = true then
    DrawSelect(Rect,CameraX);
end;

procedure TPlace.Init(x, y, w, h: integer; idPlace: byte);
begin
  Rect:= ReturnRect(x,y,w,h);
  ID:= idPlace;
  Active:= false;
  Check:= false;
  Enable:= true;
end;

procedure TPlace.Process(flagSelect: integer);
begin
  if Enable = false then
    exit;
  if flagSelect <> 0 then
    exit;
  if PointInRect(CursorX+CameraMainX,CursorY,Rect) then
  begin
    if (GetMouseButtonPressed = MB_LEFT) and (MouseDown = MB_NONE) then
    begin
      if Active = true then
        Active:= false
      else
        Active:= true;
    end;
  end;
end;

procedure LoadLevel(numLevel: integer);
var numElement: integer;
    F: file;
begin
  if not FileExists('level\level'+inttostr(numLevel)+'.lvl') then
    exit;
  Assign(F, 'level\level'+inttostr(numLevel)+'.lvl');
  Reset(F,1);
  SetLength(Levels[numLevel].Ground,0);
  SetLength(Levels[numLevel].Trap,0);
  SetLength(Levels[numLevel].Bonus,0);
  SetLength(Levels[numLevel].Place,0);
  SetLength(Levels[numLevel].EnemyLevel,0);
  Levels[numLevel].countGround:= 0;
  Levels[numLevel].countTrap:= 0;
  Levels[numLevel].countBonus:= 0;
  Levels[numLevel].countPlace:= 0;
  Levels[numLevel].countEnemy:= 0;
  BlockRead(F,Levels[numLevel].Width, SizeOf(Levels[numLevel].Width));
  BlockRead(F,Levels[numLevel].countGround, SizeOf(Levels[numLevel].countGround));
  SetLength(Levels[numLevel].Ground,Levels[numLevel].countGround);
  for numElement:= 0 to Levels[numLevel].countGround-1 do
  begin
    Levels[numLevel].Ground[numElement]:= TGround.Create;
    BlockRead(F,Levels[numLevel].Ground[numElement].Rect, SizeOf(Levels[numLevel].Ground[numElement].Rect));
    BlockRead(F,Levels[numLevel].Ground[numElement].numTex, SizeOf(Levels[numLevel].Ground[numElement].numTex));
    BlockRead(F,Levels[numLevel].Ground[numElement].Active, SizeOf(Levels[numLevel].Ground[numElement].Active));
    BlockRead(F,Levels[numLevel].Ground[numElement].Fon, SizeOf(Levels[numLevel].Ground[numElement].Fon));
    BlockRead(F,Levels[numLevel].Ground[numElement].Speed, SizeOf(Levels[numLevel].Ground[numElement].Speed));
    BlockRead(F,Levels[numLevel].Ground[numElement].StartX, SizeOf(Levels[numLevel].Ground[numElement].StartX));
    BlockRead(F,Levels[numLevel].Ground[numElement].StartY, SizeOf(Levels[numLevel].Ground[numElement].StartY));
    BlockRead(F,Levels[numLevel].Ground[numElement].DeltaXRigth, SizeOf(Levels[numLevel].Ground[numElement].DeltaXRigth));
    BlockRead(F,Levels[numLevel].Ground[numElement].DeltaYUp, SizeOf(Levels[numLevel].Ground[numElement].DeltaYUp));
    BlockRead(F,Levels[numLevel].Ground[numElement].Vector, SizeOf(Levels[numLevel].Ground[numElement].Vector));
  end;
  BlockRead(F,Levels[numLevel].countTrap, SizeOf(Levels[numLevel].countTrap));
  SetLength(Levels[numLevel].Trap,Levels[numLevel].countTrap);
  for numElement:= 0 to Levels[numLevel].countTrap-1 do
  begin
    Levels[numLevel].Trap[numElement]:= TTrap.Create;
    BlockRead(F,Levels[numLevel].Trap[numElement].Rect, SizeOf(Levels[numLevel].Trap[numElement].Rect));
    BlockRead(F,Levels[numLevel].Trap[numElement].Id, SizeOf(Levels[numLevel].Trap[numElement].Id));
    BlockRead(F,Levels[numLevel].Trap[numElement].Active, SizeOf(Levels[numLevel].Trap[numElement].Active));
    BlockRead(F,Levels[numLevel].Trap[numElement].Angle, SizeOf(Levels[numLevel].Trap[numElement].Angle));
    BlockRead(F,Levels[numLevel].Trap[numElement].SpeedX, SizeOf(Levels[numLevel].Trap[numElement].SpeedX));
    BlockRead(F,Levels[numLevel].Trap[numElement].FrameNum, SizeOf(Levels[numLevel].Trap[numElement].FrameNum));
  end;
  BlockRead(F,Levels[numLevel].countBonus, SizeOf(Levels[numLevel].countBonus));
  SetLength(Levels[numLevel].Bonus,Levels[numLevel].countBonus);
  for numElement:= 0 to Levels[numLevel].countBonus-1 do
  begin
    Levels[numLevel].Bonus[numElement]:= TBonus.Create;
    BlockRead(F,Levels[numLevel].Bonus[numElement].Rect, SizeOf(Levels[numLevel].Bonus[numElement].Rect));
    BlockRead(F,Levels[numLevel].Bonus[numElement].Id, SizeOf(Levels[numLevel].Bonus[numElement].Id));
    BlockRead(F,Levels[numLevel].Bonus[numElement].Active, SizeOf(Levels[numLevel].Bonus[numElement].Active));
    BlockRead(F,Levels[numLevel].Bonus[numElement].Angle, SizeOf(Levels[numLevel].Bonus[numElement].Angle));
    BlockRead(F,Levels[numLevel].Bonus[numElement].Enable, SizeOf(Levels[numLevel].Bonus[numElement].Enable));
  end;
  BlockRead(F,Levels[numLevel].countPlace, SizeOf(Levels[numLevel].countPlace));
  SetLength(Levels[numLevel].Place,Levels[numLevel].countPlace);
  for numElement:= 0 to Levels[numLevel].countPlace-1 do
  begin
    Levels[numLevel].Place[numElement]:= TPlace.Create;
    Levels[numLevel].Place[numElement].Init(0,0,0,0,0);
    BlockRead(F,Levels[numLevel].Place[numElement].Rect, SizeOf(Levels[numLevel].Place[numElement].Rect));
    BlockRead(F,Levels[numLevel].Place[numElement].ID, SizeOf(Levels[numLevel].Place[numElement].ID));
    BlockRead(F,Levels[numLevel].Place[numElement].Active, SizeOf(Levels[numLevel].Place[numElement].Active));
    BlockRead(F,Levels[numLevel].Place[numElement].Check, SizeOf(Levels[numLevel].Place[numElement].Check));
  end;
  BlockRead(F,Levels[numLevel].countEnemy, SizeOf(Levels[numLevel].countEnemy));
  SetLength(Levels[numLevel].EnemyLevel,Levels[numLevel].countEnemy);
  for numElement:= 0 to Levels[numLevel].countEnemy-1 do
  begin
    Levels[numLevel].EnemyLevel[numElement]:= TEnemy.Create;
    Levels[numLevel].EnemyLevel[numElement].Init(0,0,0,0,0);
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Rect, SizeOf(Levels[numLevel].EnemyLevel[numElement].Rect));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Frame, SizeOf(Levels[numLevel].EnemyLevel[numElement].Frame));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].ID, SizeOf(Levels[numLevel].EnemyLevel[numElement].ID));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Active, SizeOf(Levels[numLevel].EnemyLevel[numElement].Active));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Health, SizeOf(Levels[numLevel].EnemyLevel[numElement].Health));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Score, SizeOf(Levels[numLevel].EnemyLevel[numElement].Score));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Speed, SizeOf(Levels[numLevel].EnemyLevel[numElement].Speed));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Vector, SizeOf(Levels[numLevel].EnemyLevel[numElement].Vector));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].TimeReload, SizeOf(Levels[numLevel].EnemyLevel[numElement].TimeReload));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].RadiusAgr, SizeOf(Levels[numLevel].EnemyLevel[numElement].RadiusAgr));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Life, SizeOf(Levels[numLevel].EnemyLevel[numElement].Life));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].Jump, SizeOf(Levels[numLevel].EnemyLevel[numElement].Jump));
    BlockRead(F,Levels[numLevel].EnemyLevel[numElement].HeightJump, SizeOf(Levels[numLevel].EnemyLevel[numElement].HeightJump));
  end;
  Close(F);
end;

end.
