unit uCollision;

interface

uses
  uShape, uPolyShape, uArbiter, uVect, math;

  function cpCollideShapes(shape1, shape2: PShape; var arr: cpContacts): integer;

implementation

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function circle2circleQuery(p1, p2: cpVect; r1, r2: single; var con: cpContacts): integer;
var
  mindist, distsq, dist, non_zero_dist: single;
  delta: cpVect;
begin
  result:= 0;
  mindist:= r1 + r2;
  delta:= cpvsub(p2, p1);
  distsq:= cpvlengthsq(delta);
	if distsq >= mindist * mindist then exit;

  dist:= sqrt(distsq);

  if dist <> 0 then non_zero_dist:= dist else non_zero_dist:= INFINITY;
  SetLength(con, 1);
  con[0]:= cpContactInit(cpvadd(p1, cpvmult(delta, 0.5 + (r1 - 0.5 * mindist) / non_zero_dist)) , cpvmult(delta, 1 / non_zero_dist), dist - mindist, 0);
  result:= 1;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function circle2circle(circ1, circ2: PCircleShape; var arr: cpContacts): integer;
begin
	Result:= circle2circleQuery(circ1.tc, circ2.tc, circ1.r, circ2.r, arr);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function circle2segment(circ: PCircleShape; seg: PSegmentShape; var con: cpContacts): integer;
var
  dn, dist, dt, dtMin, dtMax: single;
  n: cpVect;
begin
  result:= 0;

  dn:= cpvdot(seg.tn, circ.tc) - cpvdot(seg.ta, seg.tn);
  dist:= abs(dn) - circ.r - seg.r;
  if dist > 0 then exit;

	dt:= -cpvcross(seg.tn, circ.tc);
	dtMin:= -cpvcross(seg.tn, seg.ta);
	dtMax:= -cpvcross(seg.tn, seg.tb);

  if dt < dtMin then
  begin
		if dt >= (dtMin - circ.r) then Result:= circle2circleQuery(circ.tc, seg.ta, circ.r, seg.r, con);
  end else
  begin
    if dt < dtMax then
    begin
      if dn < 0 then n:= seg.tn else n:= cpvneg(seg.tn);
      SetLength(con, 1);
      con[0]:= cpContactInit(cpvadd(circ.tc, cpvmult(n, circ.r + dist * 0.5)), n, dist, 0);
      result:= 1;
    end else
    begin
			if dt < (dtMax + circ.r) then	Result:= circle2circleQuery(circ.tc, seg.tb, circ.r, seg.r, con);
    end;
  end;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function segValueOnAxis(seg: PSegmentShape; n: cpVect; d: single): single;
var
  a, b: single;
begin
	a:= cpvdot(n, seg.ta) - seg.r;
	b:= cpvdot(n, seg.tb) - seg.r;
	Result:= min(a, b) - d;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure findPointsBehindSeg(var con: cpContacts; var num: integer; seg: PSegmentShape; poly: PPolyShape; pDist, coef: single);
var
  i: integer;
  dta, dtb, dt: single;
  n, v: cpVect;
begin
	dta:= cpvcross(seg.tn, seg.ta);
	dtb:= cpvcross(seg.tn, seg.tb);
	n:= cpvmult(seg.tn, coef);

  for i:= 0 to poly.numVerts - 1 do
  begin
    v:= poly.tVerts[i];
    if(cpvdot(v, n) < cpvdot(seg.tn, seg.ta) * coef + seg.r) then
    begin
      dt:= cpvcross(seg.tn, v);
      if (dta >= dt) and (dt >= dtb) then
      begin
        num:= num + 1;
        SetLength(con, num);
        con[num - 1]:= cpContactInit(v, n, pDist, integer(poly) * 123 + i * 321);
      end;
    end
  end;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function seg2poly(seg: PSegmentShape; poly: PPolyShape; var arr: cpContacts): integer;
const
  cp_collision_slop = 0.1;
var
  axes: array of cpPolyShapeAxis;
  i, mini, num: integer;
  segD, minNorm, minNeg, poly_min, dist: single;
  poly_n, va, vb: cpVect;
begin
  Result:= 0;

  SetLength(axes, poly.numVerts);
  for i:= 0 to poly.numVerts - 1 do
    axes[i]:= poly.tAxes[i];

  segD:= cpvdot(seg.tn, seg.ta);

	minNorm:= cpPolyShapeValueOnAxis(poly, seg.tn, segD) - seg.r;
	minNeg:= cpPolyShapeValueOnAxis(poly, cpvneg(seg.tn), -segD) - seg.r;
	if (minNeg > 0) or (minNorm > 0) then exit;

  mini:= 0;
  poly_min:= segValueOnAxis(seg, axes[0].n, axes[0].d);
  if poly_min > 0 then Exit;
  for i:= 0 to poly.numVerts - 1 do
  begin
    dist:= segValueOnAxis(seg, axes[i].n, axes[i].d);
    if dist > 0 then exit
      else if dist > poly_min then
        begin
          poly_min:= dist;
          mini:= i;
        end
  end;

	num:= 0;
  poly_n:= cpvneg(axes[mini].n);

	va:= cpvadd(seg.ta, cpvmult(poly_n, seg.r));
	vb:= cpvadd(seg.tb, cpvmult(poly_n, seg.r));

	if cpPolyShapeContainsVert(poly, va) then
  begin
    num:= num + 1;
    SetLength(arr, num);
		arr[num - 1]:= cpContactInit(va, poly_n, poly_min, integer(seg) * 123 + integer(poly) * 321);
  end;
	if cpPolyShapeContainsVert(poly, vb) then
  begin
    num:= num + 1;
    SetLength(arr, num);
		arr[num - 1]:= cpContactInit(vb, poly_n, poly_min, integer(seg) * 321 + integer(poly) * 123);
  end;

	poly_min := poly_min - cp_collision_slop;
	if (minNorm >= poly_min) or (minNeg >= poly_min) then
  begin
		if minNorm > minNeg  then findPointsBehindSeg(arr, num, seg, poly, minNorm, 1)
		  else findPointsBehindSeg(arr, num, seg, poly, minNeg, -1);
  end;

  Result:= num;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function findMSA(poly: PPolyShape; axes: array of cpPolyShapeAxis; num: integer; var min_out: single): integer;
var
  i, min_index: integer;
  mmin, dist: single;
begin
  min_index:= 0;
  min_out:= 0;
	mmin:= cpPolyShapeValueOnAxis(poly, axes[0].n, axes[0].d);
  Result:= -1;
	if mmin > 0 then exit;

  for i:= 1 to num - 1 do
  begin
    dist:= cpPolyShapeValueOnAxis(poly, axes[i].n, axes[i].d);
    if dist > 0 then exit
      else if dist > mmin then
        begin
          mmin:= dist;
          min_index:= i;
        end;
  end;

	min_out:= mmin;
	Result:= min_index;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function findVerts(var con: cpContacts; const poly1, poly2: PPolyShape; n: cpVect; dist: single): integer;
var
  i, num: integer;
  v: cpVect;
begin
	num:= 0;

 	for i:= 0 to poly1.numVerts - 1 do
  begin
    v:= poly1.tVerts[i];
		if cpPolyShapeContainsVert(poly2, v) then
    begin
      num:= num + 1;
      SetLength(con, num);
      con[num - 1]:= cpContactInit(v, n, dist, integer(poly1) * 123 + i * 321);
    end;
  end;

 	for i:= 0 to poly2.numVerts - 1 do
  begin
    v:= poly2.tVerts[i];
		if cpPolyShapeContainsVert(poly1, v) then
    begin
      num:= num + 1;
      SetLength(con, num);
      con[num - 1]:= cpContactInit(v, n, dist, integer(poly2) * 321 + i * 123);
    end;
  end;

	Result:= num;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function poly2poly(poly1, poly2: PPolyShape; var arr: cpContacts): integer;
var
  min1, min2: single;
  mini1, mini2: integer;
begin
  result:= 0;

	mini1:= findMSA(poly2, poly1.tAxes, poly1.numVerts, min1);
	if mini1 = -1 then exit;
	mini2:= findMSA(poly1, poly2.tAxes, poly2.numVerts, min2);
	if mini2 = -1 then exit;

  if min1 > min2 then	Result:= findVerts(arr, poly1, poly2, poly1.tAxes[mini1].n, min1)
  	else Result:= findVerts(arr, poly1, poly2, cpvneg(poly2.tAxes[mini2].n), min2);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function circle2poly(circ: PCircleShape; poly: PPolyShape; var con: cpContacts): integer;
var
  axes: array of cpPolyShapeAxis;
  i, mini: integer;
  mmin, dist, dta, dtb, dt: single;
  n, a, b: cpVect;
begin
  Result:= 0;

  SetLength(axes, poly.numVerts);
  for i:= 0 to poly.numVerts - 1 do
    axes[i]:= poly.tAxes[i];

	mini:= 0;
	mmin:= cpvdot(axes[0].n, circ.tc) - axes[0].d - circ.r;
  for i:= 0 to poly.numVerts - 1 do
  begin
    dist:= cpvdot(axes[i].n, circ.tc) - axes[i].d - circ.r;
    if dist > 0 then exit
      else if dist > mmin then
        begin
          mmin:= dist;
          mini:= i;
        end;
  end;

	n:= axes[mini].n;
	a:= poly.tVerts[mini];
	b:= poly.tVerts[(mini + 1) mod poly.numVerts];
	dta:= cpvcross(n, a);
	dtb:= cpvcross(n, b);
	dt:= cpvcross(n, circ.tc);
		
  if dt < dtb then Result:= circle2circleQuery(circ.tc, b, circ.r, 0, con)
    else if dt < dta then
      begin
        SetLength(con, 1);
        con[0]:= cpContactInit(cpvsub(circ.tc, cpvmult(n, circ.r + mmin / 2)), cpvneg(n), mmin, 0);
        Result:= 1;
      end else Result:= circle2circleQuery(circ.tc, a, circ.r, 0, con);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpCollideShapes(shape1, shape2: PShape; var arr: cpContacts): integer;
begin
  Result:= 0;
  if (shape1.ShapeType = CP_CIRCLE_SHAPE) and (shape2.ShapeType = CP_CIRCLE_SHAPE) then Result:= circle2circle(shape1.DfShape, shape2.DfShape, arr);
  if (shape1.ShapeType = CP_CIRCLE_SHAPE) and (shape2.ShapeType = CP_SEGMENT_SHAPE) then Result:= circle2segment(shape1.DfShape, shape2.DfShape, arr);


  if (shape1.ShapeType = CP_CIRCLE_SHAPE) and (shape2.ShapeType = CP_POLY_SHAPE) then Result:= circle2Poly(shape1.DfShape, shape2.DfShape, arr);

  if (shape1.ShapeType = CP_SEGMENT_SHAPE) and (shape2.ShapeType = CP_POLY_SHAPE) then Result:= seg2poly(shape1.DfShape, shape2.DfShape, arr);

  if (shape1.ShapeType = CP_POLY_SHAPE) and (shape2.ShapeType = CP_POLY_SHAPE) then Result:= poly2poly(shape1.DfShape, shape2.DfShape, arr);
end;

end.
