unit uPolyShape;

interface

uses
  uBody, uShape, uVect, uBB, math;

type
  cpPolyShapeAxis = record
	  n: cpVect;
	  d: single;
  end;

type
  PPolyShape = ^TPolyShape;
  TPolyShape = record
    shape: PShape;
    numVerts: integer;
    verts, tVerts: array of cpVect;
    axes, tAxes: array of cpPolyShapeAxis;
    Next, Prev: PPolyShape;
  end;

var
  cpPolyShape: PPolyShape;
  PolyShapesCount: integer;

  function cpPolyShapeValueOnAxis(poly: PPolyShape; n: cpVect; d: single): single;
  function cpPolyShapeContainsVert(poly: PPolyShape; v: cpVect): boolean;
  function cpPolyShapeCacheData(poly: PPolyShape; p, rot: cpVect): cpBB;
  function cpPolyShapeCreate(body: PBody; numVerts: integer; verts: array of cpVect; offset: cpVect): PShape;

implementation

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpPolyShapeValueOnAxis(poly: PPolyShape; n: cpVect; d: single): single;
var
  i: integer;
  mmin: single;
begin
	mmin:= cpvdot(n, poly.tVerts[0]);
	
  for i:= 1 to poly.numVerts - 1 do
    mmin:= min(mmin, cpvdot(n, poly.tVerts[i]));

	Result:= mmin - d;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpPolyShapeContainsVert(poly: PPolyShape; v: cpVect): boolean;
var
  i: integer;
  dist: single;
begin
  Result:= false;
  for i:= 0 to poly.numVerts - 1 do
  begin
		dist:= cpvdot(poly.tAxes[i].n, v) - poly.tAxes[i].d;
		if dist > 0 then exit;
  end;
	Result:= true;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure cpPolyShapeTransformVerts(poly: PPolyShape; p, rot: cpVect);
var
  i: integer;
begin
  for i:= 0 to poly.numVerts - 1 do
    poly.tVerts[i]:= cpvadd(p, cpvrotate(poly.verts[i], rot));
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
procedure cpPolyShapeTransformAxes(poly: PPolyShape; p, rot: cpVect);
var
  i: integer;
  n: cpVect;
begin
  for i:= 0 to poly.numVerts - 1 do
  begin
    n:= cpvrotate(poly.axes[i].n, rot);
    poly.tAxes[i].n:= n;
    poly.tAxes[i].d:= cpvdot(p, n) + poly.axes[i].d;
  end;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpPolyShapeCacheData(poly: PPolyShape; p, rot: cpVect): cpBB;
var
  i: integer;
  l, b, r, t: single;
  v: cpVect;
begin
	cpPolyShapeTransformAxes(poly, p, rot);
	cpPolyShapeTransformVerts(poly, p, rot);

	l:= poly.tVerts[0].x;
  r:= poly.tVerts[0].x;
	b:= poly.tVerts[0].y;
  t:= poly.tVerts[0].y;

  for i:= 0 to poly.numVerts - 1 do
  begin
		v:= poly.tVerts[i];

		l:= min(l, v.x);
		r:= max(r, v.x);

		b:= min(b, v.y);
		t:= max(t, v.y);
	end;
	
	Result:= cpBBNew(l, b, r, t);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpPolyShapeCreate(body: PBody; numVerts: integer; verts: array of cpVect; offset: cpVect): PShape;
var
  s: PShape;
  sd: PPolyShape;
  i: integer;
  a, b, n: cpVect;
begin
  New(sd);

  s:= cpShapeCreate(CP_POLY_SHAPE, body);
  s.DfShape:= sd;
  s.body:= body;

  sd.numVerts:= numVerts;
  SetLength(sd.verts, numVerts);
  SetLength(sd.tVerts, numVerts);
  SetLength(sd.axes, numVerts);
  SetLength(sd.tAxes, numVerts);
  for i:= 0 to numVerts - 1 do
  begin
    a:= cpvadd(offset, verts[i]);
    b:= cpvadd(offset, verts[(i + 1) mod sd.numVerts]);
    n:= cpvnormalize(cpvperp(cpvsub(b, a)));

    sd.verts[i]:= a;
    sd.axes[i].n:= n;
    sd.axes[i].d:= cpvdot(n, a);
  end;

  updateBB(s);
  Result:= s;
end;

end.
