unit uVect;

interface

uses
  math;

type
  cpVect = record
    x, y: single;
  end;

  function cpv(const x, y: single): cpVect;
  function cpvadd(const v1, v2: cpVect): cpVect;
  function cpvneg(const v: cpVect): cpVect;
  function cpvsub(const v1, v2: cpVect): cpVect;
  function cpvmult(const v: cpVect; const s: single): cpVect;
  function cpvdot(const v1, v2: cpVect): single;
  function cpvcross(const v1, v2: cpVect): single;
  function cpvperp(const v: cpVect): cpVect;
  function cpvrperp(const v: cpVect): cpVect;
  function cpvproject(const v1, v2: cpVect): cpVect;
  function cpvrotate(const v1, v2: cpVect): cpVect;
  function cpvunrotate(const v1, v2: cpVect): cpVect;
  function cpvlength(const v: cpVect): single;
  function cpvlengthsq(const v: cpVect): single;
  function cpvnormalize(const v: cpVect): cpVect;
  function cpvforangle(const a: single): cpVect;  
  function cpvtoangle(const v: cpVect): single;

const
  cpvzero : cpVect = (x : 0.0; y : 0.0);
implementation

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpv(const x, y: single): cpVect;
begin
  Result.x:= x;
  Result.y:= y;  
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvadd(const v1, v2: cpVect): cpVect;
begin
  Result:= cpv(v1.x + v2.x, v1.y + v2.y);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvneg(const v: cpVect): cpVect;
begin
  Result:= cpv(-v.x, -v.y);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvsub(const v1, v2: cpVect): cpVect;
begin
	Result:= cpv(v1.x - v2.x, v1.y - v2.y);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvmult(const v: cpVect; const s: single): cpVect;
begin
  Result:= cpv(v.x * s, v.y * s);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvdot(const v1, v2: cpVect): single;
begin
	Result:= v1.x * v2.x + v1.y * v2.y;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvcross(const v1, v2: cpVect): single;
begin
	Result:= v1.x * v2.y - v1.y * v2.x;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvperp(const v: cpVect): cpVect;
begin
  Result:= cpv(-v.y, v.x);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvrperp(const v: cpVect): cpVect;
begin
	Result:= cpv(v.y, -v.x);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvproject(const v1, v2: cpVect): cpVect;
begin
	Result:= cpvmult(v2, cpvdot(v1, v2)/cpvdot(v2, v2));
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvrotate(const v1, v2: cpVect): cpVect;
begin
  Result:= cpv(v1.x * v2.x - v1.y * v2.y, v1.x * v2.y + v1.y * v2.x);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvunrotate(const v1, v2: cpVect): cpVect;
begin
  Result:= cpv(v1.x * v2.x + v1.y * v2.y, v1.y * v2.x - v1.x * v2.y);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvlength(const v: cpVect): single;
begin
  Result:= sqrt(cpvdot(v, v));
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvlengthsq(const v: cpVect): single;
begin
  Result:= cpvdot(v, v);
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvnormalize(const v: cpVect): cpVect;
begin
  Result:= cpvmult(v, 1 / cpvlength(v));
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvforangle(const a: single): cpVect;
begin
  Result:= cpv(cos(a), sin(a));
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
function cpvtoangle(const v: cpVect): single;
begin
  Result:= ArcTan2(v.x, v.y);
end;

end.
