unit NFAFont;
{
  by Hex[man]
  Date: 27.04.2009
  Update: 05.01.2010

  Simple Font Engine
}
interface
uses Windows, OpenGL, sNail;

type
  TNFASymbol = record
    l, t, w, h: single;
    size: byte;
  end;

  TNFAData = record
    count: byte;
    start, finish: byte;
    symbHeight: byte;
    symbols:array of TNFASymbol;
  end;
  
  PNFAFont = ^TNFAFont;
  TNFAFont = object
  private
    data: TNFAData;
    texfont: cardinal;
    fkerning: shortint;
    function CheckSymbol(ch: char): boolean;
  public
    procedure Load(const fileName:string; kerning: shortint = -1);
    procedure SetTexture(const texture: cardinal);
    procedure Render(x, y: single; str: string);

    function StrWidth(const str: string): word;
    function StrHeight: byte;
    function SymbolWidth(const symb: char):byte;
    procedure Free;
  end;

implementation

{ TNFAFont }

procedure TNFAFont.Load(const fileName: string; kerning: shortint = -1);
var
  f: file of byte;
begin
  ZeroMemory(@data, sizeof(data));
  AssignFile(f, FileName);
  Reset(f);
  with data do
  begin
    BlockRead(f, count, sizeof(count));
    BlockRead(f, start, sizeof(start));
    BlockRead(f, finish, sizeof(finish));
    BlockRead(f, symbHeight, sizeof(symbHeight));
    SetLength(symbols, count);
    BlockRead(f, symbols[0], sizeof(TNFASymbol) * count);
  end;
  CloseFile(f);

  fkerning := kerning;
end;

function TNFAFont.CheckSymbol(ch: char): boolean;
begin
  Result := ((ord(ch) >= data.start) and (ord(ch) <= data.finish));
end;

procedure TNFAFont.Render(x, y: single; str: string);
var
  i: word;
  w2 ,h2: single;
  xpos, ypos, symbsize: word;
  id: byte;
begin
  gl_TexEnable(texfont);
  h2 := (data.symbHeight / 2);
  xpos := 0;
  ypos := 0;

  for i := 1 to Length(Str) do
  begin
    if str[i] = #13 then
    begin
      xpos := 0;
      inc(ypos, data.symbHeight);
      continue;
    end;

    if not CheckSymbol(str[i]) then continue;
    
    id := ord(str[i]) - data.start;
    symbsize := data.symbols[id].size;
    w2 := (symbsize / 2);

    glPushMatrix;
      glTranslatef(x + w2 + xpos, y + h2 + ypos, 0);
      glBegin(GL_QUADS);
        with data.symbols[id] do
        begin
          glTexCoord2f(l, t); glVertex2f(-w2, -h2);
          glTexCoord2f(w, t); glVertex2f(w2, -h2);
          glTexCoord2f(w, h); glVertex2f(w2, h2);
          glTexCoord2f(l, h); glVertex2f(-w2, h2);
        end;
      glEnd;
    glPopMatrix;

    inc(xpos, symbsize + fkerning);
  end;
  gl_TexDisable;
end;

procedure TNFAFont.SetTexture(const texture: cardinal);
begin
  texfont := texture;
end;

procedure TNFAFont.Free;
begin
end;

function TNFAFont.StrWidth(const str: string): word;
var
  i, totalX: word;
begin
  totalX:=0;
  for i:=1 to Length(Str) do
    if CheckSymbol(str[i]) then
      inc(totalX, data.symbols[ord(str[i]) - data.start].size + fkerning);
  Result:=totalX;
end;

function TNFAFont.StrHeight: byte;
begin
  Result := data.symbHeight;
end;

function TNFAFont.SymbolWidth(const symb: char):byte;
begin
  if CheckSymbol(symb) then
    Result := data.symbols[ord(symb) - data.start].size
  else
    Result := 0;
end;

end.
