unit uParticles;

interface
uses sNail, Windows, OpenGL, uUtils, Math;
const
  PART_SNOWFLAKE = 1;
  PART_SMOKE = 2;
  PART_SPARKS = 3;
  PART_GOLDSNOW = 4;
  PART_GOLDCIRCLE = 5;

type
  TPartObject = record
    p: TVector;
    life: single;
    speed: single;
    angle: single;
    alpha: single;
    power: single;
    size: single;
  end;

  TSnowFlake = record
    p: TVector;
    alpha: single;
    afactor : single;
    speed: single;
    way: single;
  end;

  TPartEmitter = class
  protected
    active: boolean;
    color: integer;
    objs: array of TPartObject;
    count: word;
    immortal: boolean;
    texture: cardinal;
  public
    constructor Create(dPos:PVector; pow:single); virtual;
    procedure SetCount(const nCount:word);
    procedure Update;
    procedure Update_part(id: byte); virtual;
    procedure Render; virtual;
    procedure Reset(id: byte); virtual;
  end;

  TPartSnowflake = class(TPartEmitter)
  public
    pos: TVector;
    constructor Create(dPos: PVector; pow: single);override;
    procedure Update_part(id: byte);override;
  end;

  TPartSmoke = class(TPartEmitter)
  public
    pos:PVector;
    constructor Create(dPos: PVector; pow:single);override;
    procedure Update_part(id: byte);override;
    procedure Reset(id: byte);override;
  end;

  TPartSnowSparks = class(TPartSnowflake)
  public
    constructor Create(dPos: PVector; pow:single);override;
  end;

  TPartGoldSnow = class(TPartSnowflake)
  public
    constructor Create(dPos: PVector; pow:single);override;
  end;

  TPartGoldCircle = class(TPartEmitter)
  public
    pos: PVector;

    constructor Create(dPos: PVector; pow: single);override;
    procedure Update_part(id: byte);override;
    procedure Reset(id: byte);override;
  end;

  TPartEngine = class
  public
    objs: array[0..63] of TPartEmitter;
    constructor Create;
    function Add(pType: byte; pos: PVector; pow:single): integer;
    procedure Delete(id: integer);
    procedure Update;
    procedure Render;
    procedure Clear;
  end;

  TSnowFlakes = object
  private
    snowflakes: array[0..100] of TSnowFlake;
  public
    procedure Init;
    procedure Render;
    procedure Update(mx: single);
  end;

var
  texSpark, texSnow: cardinal;
  parts: TPartEngine;

implementation
uses uGame, uBody, uLevel;
{ TParticleEmitter }

procedure TPartEmitter.SetCount(const nCount: word);
begin
  SetLength(objs,nCount);
  count:=High(objs);
end;

constructor TPartEmitter.Create(dPos:PVector; pow:single);
begin
  inherited Create;
  color := $ffffff;
  active := true;
end;

procedure TPartEmitter.Render;
var
  i:byte;
begin
  gl_TexEnable(texture);
  for i:=0 to count-1 do
    with objs[i] do
      if life > 0 then
      begin
        glPushMatrix;
          gl_SetColor(color, Round(Alpha));
          gl_RenderTex(p.x - size / 2,p.y - size / 2, Round(size), Round(size), Round(RadToDeg(angle)));
        glPopMatrix;
      end;
  gl_TexDisable;
end;

procedure TPartEmitter.Reset(id: byte);
begin
end;

procedure TPartEmitter.Update_part(id: byte);
begin
end;

procedure TPartEmitter.Update;
var
  i: byte;
begin
  inherited;
  for i := 0 to count-1 do
    with objs[i] do
    begin
      if life > 0 then
      begin
        life := life - 1.0;
        Update_part(i);
      end else
        if immortal then
          Reset(i)
        else
          active:=false;
    end;
end;

{ TPartEngine ------------------------------------------}

constructor TPartEngine.Create;
begin
  ZeroMemory(@Objs,sizeof(Objs));
end;

function TPartEngine.Add(pType: byte; pos: PVector; pow: single): integer;
var
  i, index: shortint;
begin
  index:=-1;
  for i := 0 to High(objs) do
    if objs[i]  = nil then
    begin
      index:=i;
      break;
    end;

  if index > -1 then
    case PType of
      PART_SNOWFLAKE: objs[index]:= TPartSnowflake.Create(pos, pow);
      PART_SMOKE: objs[index]:= TPartSmoke.Create(pos, pow);
      PART_SPARKS: objs[index]:= TPartSnowSparks.Create(pos, pow);
      PART_GOLDSNOW : objs[index]:= TPartGoldSnow.Create(pos, pow);
      PART_GOLDCIRCLE : objs[index]:= TPartGoldCircle.Create(pos, pow);
    end;
  Result := index;
end;

procedure TPartEngine.Delete(id: integer);
begin
  objs[id].Free;
  objs[id]:=nil;
end;

procedure TPartEngine.Render;
var
  i: byte;
begin
  gl_SetBlend(BLEND_ADD);
  for i := 0 to high(Objs) do
    if (objs[i] <> nil) then
      objs[i].Render;
  gl_SetBlend(BLEND_SUB);
end;

procedure TPartEngine.Update;
var
  i: byte;
begin
  for i := 0 to High(objs) do
    if (objs[i] <> nil) then
    begin
      if not objs[i].active then
        Delete(i)
      else
        objs[i].Update;
    end;
end;

{ TBoom }

constructor TPartSnowflake.Create(dPos: PVector; pow: single);
var
  i:byte;
begin
  inherited;
  SetCount(3);
  immortal := false;
  pos := dPos^;
  texture := texSnow;

  for i := 0 to count - 1 do
    with objs[i] do
    begin
      p := pos;
      life := 42;
      alpha := 255;
      angle:=random * pi * 2;
      speed:=random * pow;
      size := 10;
    end;
end;

procedure TPartSnowflake.Update_part(id: byte);
begin
  inherited;
  with objs[id] do
  begin
    p.x := p.x + sin(angle) * speed;
    p.y := p.y + cos(angle) * speed;
    alpha := alpha - 2.5;
    size := size - 0.14;
  end;
end;

{ TPartFire }

constructor TPartSmoke.Create(dPos: PVector; pow: single);
var
  i:byte;
begin
  inherited;
  SetCount(5);
  immortal := true;
  texture := texBonus;
  pos := dPos;
  for i := 0 to High(objs) do Reset(i);
end;

procedure TPartSmoke.Reset(id: byte);
begin
  inherited;
  with Objs[id] do
  begin
    p:=Pos^;
    life:= id * 5;
    alpha := 128;
    size := 6;
    angle := random * 5;
  end;
end;

procedure TPartSmoke.Update_part(id: byte);
begin
  inherited;
  with objs[id] do
  begin
    p.x := p.x + sin(-angle * 2);
    alpha := alpha - 3.2;
    size := size + 0.15;
  end;
end;

{ TPartSparks }

constructor TPartSnowSparks.Create(dPos: PVector; pow: single);
var
  i:word;
begin
  inherited;
  SetCount(10);
  texture := texBonus;

  for i := 0 to count - 1 do
    with objs[i] do
    begin
      p := pos;
      life := 55;
      alpha := 145;
      size := 24;
      angle := random * PI * 2;
      speed := random * pow;
    end;
end;

{ TPartGoldSnow }

constructor TPartGoldSnow.Create(dPos: PVector; pow: single);
begin
  inherited;
  texture := texSpark;
  color := $ff6e00;
end;

{ TPartGoldCircle }

constructor TPartGoldCircle.Create(dPos: PVector; pow: single);
var
  i: byte;
begin
  inherited;
  SetCount(15);
  immortal := true;
  texture := texSpark;
  color := $eabf3b;
  pos := dPos;

  for i := 0 to High(objs) do Reset(i);
end;

procedure TPartGoldCircle.Reset(id: byte);
begin
  inherited;
  with Objs[id] do
  begin
    p:=Pos^;
    life:= 55 + (id * 2);
    size := 18;
    alpha := random(128) + 127;
    angle := 0;
  end;
end;

procedure TPartGoldCircle.Update_part(id: byte);
begin
  inherited;
  with objs[id] do
  begin
    p.x := p.x + sin(angle);
    p.y := p.y + cos(angle);
    alpha := alpha - 3.2;
    angle := angle + 0.11;
  end;
end;

{ TSnowFlakes }

procedure TSnowFlakes.Init;
var
  i: word;
begin
  for i := 0 to High(snowflakes) do
    with snowflakes[i] do
    begin
      p.x := random(gl_GetResX);
      p.y := random(gl_GetResY);
      alpha := random(100) + 100;
      afactor := random;
      way := (random - 0.5) * 2 ;
      speed := random + 0.2;
    end;
end;

procedure TSnowFlakes.Render;
var
  i: word;
begin
  gl_SetBlend(BLEND_ADD);
  glBegin(GL_POINTS);
    for i := 0 to High(snowflakes) - 1 do
      with snowflakes[i] do
      begin
        gl_SetColor($ffffff,  Round(alpha));
        glVertex2f(p.x, p.y);
      end;
   glEnd;
   gl_TexDisable;
   gl_SetBlend(BLEND_SUB);
end;

procedure TSnowFlakes.Update(mx: single);
var
  i: word;
begin
  for i := 0 to High(snowflakes) do
    with snowflakes[i] do
    begin
      afactor := afactor + 0.02;
      p.x := p.x + (cos(afactor) / 2 ) - way;
      if p.x > gl_GetResX then p.x := 0;
      if p.x < 0 then p.x := gl_GetResX;
      p.y := p.y + speed;
      alpha := alpha - 0.2;

      if (p.y > gl_GetResY) or (alpha < 10) then
      begin
        p.y := 0;
        p.x := random(gl_GetResX);
        alpha := random(155) + 100;
      end;
    end;
end;

procedure TPartEngine.Clear;
var
  i: byte;
begin
  for i := 0 to High(objs) do
    objs[i] := nil;
end;

initialization
  parts := TPartEngine.Create;
finalization
  parts.Free;

end.
