unit uUtils;

interface
uses sNail, uVect;
type
  PVector = ^TVector;
  TVector = record
    x, y : double;
  end;

  TConfig = record
    Resolution: byte;
    ResX, ResY : word;
    FullScreen : boolean;
    PointSize, LineWidth: single;
    BgFx: boolean;
    sEnable: boolean;
    Volume: byte;
    Music: boolean; 
  end;

const
  veczero : TVector = (x : 0.0; y : 0.0);
  RES_N = 4;
  RES_X: array[0..RES_N] of word = (800, 1024, 1280, 1280, 1440);
  RES_Y: array[0..RES_N] of word = (600, 768, 1024, 800, 900);
  
var
  cfg: TConfig;

procedure LoadTexFromList(FileName: string; target:pointer);
function SetVector(x, y: double): TVector;
function Vect2cpv(v: TVector): cpVect;
function cpv2Vect(v: cpVect): TVector;
function VectDist(v1, v2: TVector): single;
function VectAdd(v1, v2: TVector): TVector;

function Numb2Str(const value: integer): string;

function ReadConfig: TConfig;
procedure WriteConfig(cfg: TConfig);

implementation

function SetVector(x, y: double):TVector;
begin
  Result.x := x;
  Result.y := y;
end;

function Vect2cpv(v: TVector):cpVect;
begin
  Result.x := v.x;
  Result.y := v.y;
end;

function cpv2Vect(v: cpVect): TVector;
begin
  Result.x := v.x;
  Result.y := v.y;
end;

function VectDist(v1, v2: TVector): single;
begin
  Result := Sqrt(Sqr(v1.x-v2.x)+Sqr(v1.y-v2.y));
end;

function VectAdd(v1, v2: TVector): TVector;
begin
  Result := SetVector(v1.x + v2.x, v1.y + v2.y)
end;

function Numb2Str(const value: integer): string;
begin
  Result := Int2Str(value);
  if (value div 10) = 0 then Result := '0' + Result;
end;

procedure LoadTexFromList(FileName: string; target:pointer);
var
  f: TextFile;
  s: string;
  proc: procedure (param : string);
begin
  AssignFile(f, FileName);
  Reset(f);

  proc := target;
  while not Eof(f) do
  begin
    ReadLn(f, s);
    proc(s);
  end;

  CloseFile(f);
end;

function ReadConfig: TConfig;
  function SetResolution(w, h: word; p, l: single): TConfig;
  begin
    Result.ResX := w;
    Result.ResY := h;
    Result.PointSize := p;
    Result.LineWidth := l;
  end;
begin
  Result.Resolution := ini_ReadInt(CONFIG_INI, 'display', 'resolution');
  case Result.Resolution of
    1 : Result := SetResolution(800, 600, 2.0, 1.0);
    2 : Result := SetResolution(1024, 768, 3.0, 2.0);
    3 : Result := SetResolution(1280, 1024, 3.5, 2.5);
    4 : Result := SetResolution(1280, 800, 3.5, 2.5);
    5 : Result := SetResolution(1440, 900, 3.5, 2.5);
    else
      Result := SetResolution(800, 600, 2.0, 1.0);
  end;

  with Result do
  begin
    FullScreen := ini_ReadBool(CONFIG_INI, 'display', 'fullscreen');
    BgFx := ini_ReadBool(CONFIG_INI, 'graphics', 'background_fx');
    Volume := ini_ReadInt(CONFIG_INI, 'sfx', 'volume');
    sEnable := ini_ReadBool(CONFIG_INI, 'sfx', 'Enable');
    Music := ini_ReadBool(CONFIG_INI, 'sfx', 'Music');
  end;
end;

procedure WriteConfig(cfg: TConfig);
begin
  with cfg do
  begin
    ini_WriteInt(CONFIG_INI, 'display', 'resolution', Resolution);
    ini_WriteBool(CONFIG_INI, 'display', 'fullscreen', FullScreen);
    ini_WriteBool(CONFIG_INI, 'graphics', 'background_fx', BgFx);
    ini_WriteInt(CONFIG_INI, 'sfx', 'Volume', Volume);
    ini_WriteBool(CONFIG_INI, 'sfx', 'Enable', sEnable);
    ini_WriteBool(CONFIG_INI, 'sfx', 'Music', Music);
  end;
end;

end.
